`timescale 1ns / 100ps
// ------------------------------------------------------------------------
// --- Module Definition
// ------------------------------------------------------------------------

module test_tbi
(
  tcg,
  phyid,
  rx_dv,
  rxd,
  rx_er,
  mdi,
  gtx_clk,
  mac_clk,
  tx_en,
  txd,
  tx_er,
  pma_rx_clk,
  rcg,
  dut_speed,
  mdc,
  mdo,
  mdoen,
  hrst,
  anx_state,
  pcs_rx_data,
  b_cerr,
  rd_err,
  an_cfg_data_vld,
  pcsrxk,
  an_cfg_data,
  rx_slip,
  rx_val
);

parameter FAMILY = 19 ;
parameter SLIP_ENABLE = 0;
// ------------------------------------------------------------------------
// --- Port Declarations
// ------------------------------------------------------------------------

input   [9:0] tcg;
input         rx_dv;
input   [7:0] rxd;
input         rx_er;
input         mdi;
output        gtx_clk;
output        mac_clk;
output        tx_en;
output  [7:0] txd;
output        tx_er;
output        pma_rx_clk;
output  reg [9:0] rcg;
output        mdc;
output        mdo;
output        mdoen;
output        hrst;
output	     rx_val;
input     [1:0] dut_speed;
input      [9:0] anx_state;
input      [4:0] phyid;
input	   [1:0] rd_err;
input	   [1:0] b_cerr;
input	   [1:0] pcsrxk;
input      [15:0] pcs_rx_data;
input	     an_cfg_data_vld;
input	     rx_slip;
input [15:0] an_cfg_data;


reg           rx_slip_r1;
reg           rx_slip_r2;
reg           rx_slip_r3;
reg           gtx_clk;
reg           mac_clk;
reg           tx_en;
reg     [7:0] txd;
reg           tx_er;
reg          pma_rx_clk;
reg           mdc;
reg           mdo;
reg           mdoen;
reg           hrst;
reg           rx_val;
reg           ewrap;
reg           anx_state_rst;
reg [9:0]  anx_latched_stat;
reg [10:0] anx_cfg_reg_cnt;
reg [10:0] mac_rx_error_det ;
reg  test_anen_en_stat; 
reg  test_rx_slip_stat; 
reg  test_1g_loopback_stat;
reg [10:0] rx_pkt_cnt;


// ------------------------------------------------------------------------
// --- Parameter Declarations
// ------------------------------------------------------------------------
parameter TP = 1;                                       // simulation delay
parameter TP5= 5;                                       // simulation delay
parameter CLK_PERIOD = 5 ;
// ------------------------------------------------------------------------
// --- Model Variables
// ------------------------------------------------------------------------
reg           chk_dis;  // Disables quescient data checking

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// clock period variables
// ------------------------------------------------------------------------
integer    prcper;
integer    gtxper ;
integer    macper ;
integer    mdcper ;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// MAC Tx variables
// ------------------------------------------------------------------------
reg    [10:0] mac_tx_mem[0:2047];
integer       mac_tx_ptr;
reg           gmii_en, gmii_end;
wire          gmii_enable, clr_gmii_en;
wire   [10:0] mac_tx_mem_data;
wire          gtx_en, gtx_er;
wire    [7:0] gtxd;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// MAC Rx variables
// ------------------------------------------------------------------------
reg    [10:0] mac_rx_mem[0:2047];
integer       mac_rx_ptr, mac_chk_ptr;
reg           mac_rcv;
reg           mac_rcvd;
reg    [10:0] mac_rx_data;


reg           mac_rx_frm;
reg           mac_rx_mii;
reg     [3:0] mac_rx_pre;
reg           mac_rx_sfd;
reg    [15:0] mac_rx_dat;
reg     [7:0] mac_rx_fls;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// MII Mgmt variables
// ------------------------------------------------------------------------
wire    [5:0] gmgmt_ctr;
reg     [5:0] mgmt_ctr;
wire          gmgmt_write, gmgmt_read;
reg           mgmt_write, mgmt_read;
wire          gmdo, gmdoen;
wire [15:0] gstat;
reg [15:0] stat;
reg           task_mgmt_write, task_mgmt_read;
reg     [4:0] mgmt_phyadr, mgmt_regadr;
reg    [15:0] mgmt_control;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// TBI shadow variables
// ------------------------------------------------------------------------
reg           phyrst, loopb; 
reg     [1:0] speed;
reg           anen, rstan;
reg           fulld;
reg           miim;
reg           drrd;
reg           dtrd;
reg [9:0] rx_slip_cnt;
reg [9:0] rcg_shift;
reg [3:0] num_of_slips;

// ------------------------------------------------------------------------
// --- Logic
// --- Clock Trees
// ------------------------------------------------------------------------


always
begin
  gtx_clk = #(gtxper/2) 1'b0;
  gtx_clk = #(gtxper/2) 1'b1;
end

always
begin
  pma_rx_clk = #(prcper/2) 1'b0;
  pma_rx_clk = #(prcper/2) 1'b1;
end

always
begin
  mac_clk = #(macper/2) 1'b0;
  mac_clk = #(macper/2) 1'b1;
end

always
begin
  mdc = #mdcper 1'b0;
  mdc = #mdcper 1'b1;
end



always @(posedge anx_state[9] or anx_state_rst or hrst)
begin
    if(anx_state_rst | hrst)
        anx_cfg_reg_cnt = 'h0;
    else 
       anx_cfg_reg_cnt = anx_cfg_reg_cnt +1;
end

always @(anx_state or anx_state_rst or hrst)
begin
    if(anx_state_rst | hrst)
        anx_latched_stat = 'h0;
    else 
        anx_latched_stat = anx_latched_stat | anx_state;
end

// ------------------------------------------------------------------------
//  gmii_en :: Receive Code Group Enable ( set in task: pma_pkt_send )
// ------------------------------------------------------------------------
//always @ ( posedge gtx_clk or posedge hrst )
always @ ( posedge mac_clk or posedge hrst )
begin
       if ( hrst )            gmii_en <= #TP 1'b0;
  else if ( clr_gmii_en )     gmii_en <= #TP 1'b0;

  gmii_end <= #TP gmii_en;
end

// ------------------------------------------------------------------------
//  mac_tx_ptr :: MAC Tx Packet Data Pointer
// ------------------------------------------------------------------------

//always @ ( posedge gtx_clk or posedge hrst )
always @ ( posedge mac_clk or posedge hrst )
begin
       if (  hrst )           mac_tx_ptr <= #TP 0;
  else if ( ~gmii_en )        mac_tx_ptr <= #TP 0;
  else if (  gmii_enable )    mac_tx_ptr <= #TP mac_tx_ptr + 1;
end

//  build "states" to control rcg mux

assign gmii_enable =  gmii_en & gmii_end;

//  watch for end of packet marker to clear rds enable
assign clr_gmii_en =  gmii_enable & mac_tx_mem_data[10];

//  get contents of packet memory
assign mac_tx_mem_data = mac_tx_mem[mac_tx_ptr];


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  gtx_en    :: Transmit Data
//  gtxd[7:0] :: Transmit Data
//  gtx_er    :: Transmit Data
// ------------------------------------------------------------------------

assign gtx_en    =       gmii_enable     & mac_tx_mem_data[8];
assign gtxd[7:0] = { 8 { gmii_enable } } & mac_tx_mem_data[7:0];
assign gtx_er    =       gmii_enable     & mac_tx_mem_data[9];

// txd[7:0] :: Transmit Data

//always @ ( posedge gtx_clk )
always @ ( posedge mac_clk )
begin
  tx_en    <= #TP gtx_en;
  txd[7:0] <= #TP gtxd[7:0];
  tx_er    <= #TP gtx_er;
end    

reg [4:0] rx_val_en_cnt;
always @ ( posedge pma_rx_clk or posedge hrst )
begin
    if(hrst)
    begin
      rx_slip_r1 = 0;
      rx_slip_r2 = 0;
      rx_slip_r3 = 0;
      rx_val = 0;
      rx_val_en_cnt = 0;
    end
    else
    begin

      // keep rx_val high for first 2 slips and next slip onwards bring down rx_val after 3 clocks.
      // rx_slip <3 rx_val = 1 //mimics iog behaviour
      // rx_slips > 3 rx_val = 0 from rx_slip_r3 = 1 to 17 clocks //mimics serdes behaviour
      if(rx_slip_cnt < 3) 
          rx_val = 1'b1;
      else if(rx_slip_r3)
          rx_val = 1'b0;
      else if(rx_val_en_cnt == 'd17)
          rx_val = 1'b1;

      // create 3 clock delayed rx_slip
      rx_slip_r3 = rx_slip_r2;
      rx_slip_r2 = rx_slip_r1;
      rx_slip_r1 = rx_slip;

      // clear the rx_val_en_cnt and bring down rx_val
      if(rx_slip_r3)
          rx_val_en_cnt = 5'h0;
      else
          rx_val_en_cnt = rx_val_en_cnt + 1'b1;
    end
end

reg rx_slip_err;
always @ ( posedge pma_rx_clk or posedge hrst )
begin
    if(hrst)
    begin
      rx_slip_cnt = 0;
      //rcg_shift = num_of_slips;
      rx_slip_err = 0;
    end
    else
    begin
      case(rcg_shift)
      0:    rcg = tcg;
      1:    rcg = {tcg[8:0],tcg[9]};
      2:    rcg = {tcg[7:0],tcg[9:8]};
      3:    rcg = {tcg[6:0],tcg[9:7]};
      4:    rcg = {tcg[5:0],tcg[9:6]};
      5:    rcg = {tcg[4:0],tcg[9:5]};
      6:    rcg = {tcg[3:0],tcg[9:4]};
      7:    rcg = {tcg[2:0],tcg[9:3]};
      8:    rcg = {tcg[1:0],tcg[9:2]};
      9:    rcg = {tcg[0],tcg[9:1]};
      10:   rcg = tcg;
      endcase
      if(rx_slip && (rcg_shift !=0))
      begin
         rcg_shift = rcg_shift -1;
         rx_slip_cnt = rx_slip_cnt+1;
      end
      else
      begin
        if(rx_slip) 
        begin
	     rx_slip_err = 1;
             rx_slip_cnt = rx_slip_cnt+1;
             $display("ERROR:unexcpected RX_SLIP :: num of slips rqrd %d & applied %d",num_of_slips,rx_slip_cnt);
        end
       end
    end
end

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ------------------------------------------------------------------------
// --- MAC Receive Logic
//  rcg[9:0] :: Receive Code Group
// ------------------------------------------------------------------------
//always @ ( posedge gtx_clk or posedge hrst )
always @ ( posedge mac_clk or posedge hrst )
begin
  if ( rx_dv | rx_er )  mac_rx_mem[mac_rx_ptr] <= #TP { 1'b1,
                                                        rx_er, 
                                                        rx_dv,
                                                        rxd[7:0] };
  if ( mac_rcv & ~( rx_dv | rx_er ) )  mac_rx_mem[mac_rx_ptr] <= #TP 11'h0;
end

//always @ ( posedge gtx_clk or posedge hrst )
always @ ( posedge mac_clk or posedge hrst )
begin
     if ( hrst )                      mac_rx_ptr <= #TP 0;
else if ( ( rx_dv | rx_er ) )                   mac_rx_ptr <= #TP mac_rx_ptr + 1;
else if ( ~( rx_dv | rx_er ) )                  mac_rx_ptr <= #TP 0;
end    

//always @ ( posedge gtx_clk or posedge hrst )
always @ ( posedge mac_clk or posedge hrst )
begin
     if ( hrst )                      mac_rcv <= #TP 1'b0;
else if ( rx_dv | rx_er )             mac_rcv <= #TP 1'b1;
else if ( ~( rx_dv | rx_er ) )        mac_rcv <= #TP 1'b0;

mac_rcvd <= #TP mac_rcv;
if ( ~mac_rcv & mac_rcvd & ~chk_dis) mac_rx_chk;
end    

// ------------------------------------------------------------------------
// --- MII Management Interface

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// gmgmt_ctr[5:0] :: Go Management Counter
assign gmgmt_ctr = { 6 { mgmt_write 
                       | mgmt_read  } } & mgmt_ctr + 6'h1;

// mgmt_ctr[5:0] :: Management Counter
always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        mgmt_ctr <= #TP 6'h0;
  else                    mgmt_ctr <= #TP gmgmt_ctr[5:0];
end


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// mgmt_write :: Management Write
assign gmgmt_write = task_mgmt_write | mgmt_write & mgmt_ctr != 6'h20;

// mgmt_write :: Management Write
always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        mgmt_write <= #TP 1'b0;
  else                    mgmt_write <= #TP gmgmt_write;
end


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// mgmt_read :: Management Read
assign gmgmt_read = task_mgmt_read | mgmt_read & mgmt_ctr != 6'h21;

// mgmt_read :: Management Read

always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        mgmt_read <= #TP 1'b0;
  else                    mgmt_read <= #TP gmgmt_read;
end

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// gmdo :: Go Management Data Output
assign gmdo = mgmt_ctr == 6'h01 & 1'b0           // ST
            | mgmt_ctr == 6'h02 & 1'b1
            | mgmt_ctr == 6'h03 & mgmt_read      // OP
            | mgmt_ctr == 6'h04 & mgmt_write
            | mgmt_ctr == 6'h05 & mgmt_phyadr[4] // PHY ADR
            | mgmt_ctr == 6'h06 & mgmt_phyadr[3]
            | mgmt_ctr == 6'h07 & mgmt_phyadr[2]
            | mgmt_ctr == 6'h08 & mgmt_phyadr[1]
            | mgmt_ctr == 6'h09 & mgmt_phyadr[0]
            | mgmt_ctr == 6'h0a & mgmt_regadr[4] // REG ADR
            | mgmt_ctr == 6'h0b & mgmt_regadr[3]
            | mgmt_ctr == 6'h0c & mgmt_regadr[2]
            | mgmt_ctr == 6'h0d & mgmt_regadr[1]
            | mgmt_ctr == 6'h0e & mgmt_regadr[0]
            | mgmt_ctr == 6'h0f & mgmt_write
            | mgmt_ctr == 6'h10 & 1'b0
            | mgmt_ctr == 6'h11 & mgmt_write & mgmt_control[15] // DATA
            | mgmt_ctr == 6'h12 & mgmt_write & mgmt_control[14]
            | mgmt_ctr == 6'h13 & mgmt_write & mgmt_control[13]
            | mgmt_ctr == 6'h14 & mgmt_write & mgmt_control[12]
            | mgmt_ctr == 6'h15 & mgmt_write & mgmt_control[11]
            | mgmt_ctr == 6'h16 & mgmt_write & mgmt_control[10]
            | mgmt_ctr == 6'h17 & mgmt_write & mgmt_control[9] 
            | mgmt_ctr == 6'h18 & mgmt_write & mgmt_control[8] 
            | mgmt_ctr == 6'h19 & mgmt_write & mgmt_control[7] 
            | mgmt_ctr == 6'h1a & mgmt_write & mgmt_control[6] 
            | mgmt_ctr == 6'h1b & mgmt_write & mgmt_control[5]
            | mgmt_ctr == 6'h1c & mgmt_write & mgmt_control[4]
            | mgmt_ctr == 6'h1d & mgmt_write & mgmt_control[3]
            | mgmt_ctr == 6'h1e & mgmt_write & mgmt_control[2]
            | mgmt_ctr == 6'h1f & mgmt_write & mgmt_control[1]
            | mgmt_ctr == 6'h20 & mgmt_write & mgmt_control[0];

// mdo :: Management Data Output
always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        mdo <= #TP5 1'b0;
  else                    mdo <= #TP5 gmdo;
end

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// gmdoen :: Go Management Data Output
assign gmdoen = ~mdoen & mgmt_ctr == 6'h01
              |  mdoen & ~( mgmt_ctr == 6'h21
                          | mgmt_ctr == 6'h0f & mgmt_read );

// mdoen :: Management Data Output Enable
always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        mdoen <= #TP5 1'b0;
  else                    mdoen <= #TP5 gmdoen;
end

wire enable = mgmt_read & mgmt_ctr >  5'h11; 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// gstat[15:0] :: Go Status
assign gstat = { 16 {    mgmt_read & mgmt_ctr > 5'h11 } } & { stat[14:0], mdi }
             | { 16 { ~( mgmt_read & mgmt_ctr > 5'h11 ) } } & { stat[15:0] };

// stat[15:0] :: Status
always @ ( posedge mdc or posedge hrst )
begin
       if ( hrst )        stat <= #TP5 16'h0;
  else                    stat <= #TP5 gstat;
end

// ------------------------------------------------------------------------
// --- Tasks
// ------------------------------------------------------------------------
// task: reset
//
task reset;
begin
  #10;  hrst  = 1'b1;
  #90;  hrst  = 1'b0;
  rst_cfg;
end
endtask

// ------------------------------------------------------------------------
// task: mgmt_wr
//
// performs a MII Mgmt Write Cycle
task mgmt_wr;
input [4:0]  phyadr, regadr;
input [15:0] control;
begin
  @ ( posedge mdc );
  #1;
  task_mgmt_write  = 1'b1;
  mgmt_phyadr = phyadr;
  mgmt_regadr = regadr;
  mgmt_control= control;
  @ ( posedge mdc );
  #1;
  task_mgmt_write  = 1'b0;
  @ ( negedge mgmt_write );
  reg_cfg(regadr, control);
end
endtask

// ------------------------------------------------------------------------
// task: mgmt_rd
//
// performs a MII Mgmt Read Cycle
task mgmt_rd;
input [4:0]  phyadr, regadr;
output [15:0] status;
begin
  @ ( posedge mdc );
  #1;
  task_mgmt_read = 1'b1;
  mgmt_phyadr = phyadr;
  mgmt_regadr = regadr;
  @ ( posedge mdc );
  #1;
  task_mgmt_read  = 1'b0;
  @ ( negedge mgmt_read );
  @ ( posedge mdc );
  #1000 status = stat;
end
endtask


// ------------------------------------------------------------------------
// task: reg_cfg
//
// updates shadow control/configuration bits
task reg_cfg;
input   [4:0] adx;
input  [15:0] data;
begin
  case( adx )
    5'h00 : begin  // Reg 0 Control
              phyrst   = data[15];  // PHY Reset
              loopb    = data[14];  // PHY Loopback
              speed[0] = data[13];  // PHY Speed Selection bit 1
              anen     = data[12];  // PHY AN Enable
              rstan    = data[9];   // PHY Restart AN
              fulld    = data[8];   // PHY Duplex ( 1=Full, 0=Half )
              speed[1] = data[6];   // PHY Speed Selection bit 2
            end
    5'h04 : begin  // Reg 4 AN Advertisement
            end
    5'h06 : begin  // Reg 6 AN Expansion Register
            end
    5'h07 : begin  // Reg 7 AN Next Page Transmit Register
            end
  endcase
end
endtask


// ------------------------------------------------------------------------
// task: rst_cfg
//
// resets shadow control/configuration bits
task rst_cfg;
begin
  phyrst   = 1'b0; // PHY Reset
  loopb    = 1'b0; // PHY Loopback
  speed[0] = 1'b0; // PHY Speed Selection bit 1
  anen     = 1'b0; // PHY AN Enable
  rstan    = 1'b0; // PHY Restart AN
  fulld    = 1'b0; // PHY Duplex ( 1=Full, 0=Half )
  speed[1] = 1'b0; // PHY Speed Selection bit 2
  miim     = 1'b0;
  dtrd     = 1'b0;
  drrd     = 1'b0;
  task_mgmt_write = 1'b0;
  task_mgmt_read  = 1'b0;
  
  mac_rx_frm = 1'b0;
  mac_rx_mii = 1'b0;
  mac_rx_pre = 4'h0;
  mac_rx_sfd = 1'b0;
  mac_rx_dat = 16'h0;
  mac_rx_fls = 8'h0;

  tx_en = 1'b0;
  txd   = 8'h0;
  tx_er = 1'b0;
  hrst  = 1'b0;
  
  rx_pkt_cnt = 0;
  ewrap = 1'b0;
  anx_state_rst = 'h0;
  anx_latched_stat = 'h0;
  anx_cfg_reg_cnt = 'h0;
  mac_rx_error_det = 'b0;
  gtx_clk = 1'b0;
  mac_clk = 1'b0;
  mdc = 1'b0;
  chk_dis = 1'b0;
  speed   = 2'b10;
  prcper  = 8;
  gtxper   = 8;
  macper   = 8;
  mdcper   = 80;
  drrd     = 1'b0;
  num_of_slips = 0;
  rcg_shift = 0;
end
endtask
initial rst_cfg;


// ------------------------------------------------------------------------
// task: mac_rx_exp
//
// sets parameters of rx event to anticipate receiving 
task mac_rx_exp;
input        mii;  // 1=frame 0=other
input        frm;  // 1=frame 0=other
input  [3:0] pre;  // # bytes
input        sfd;  // 1=sfd 0=missing
input [15:0] dat;  // # bytes
input  [7:0] fls;  // false carrier data encoding
begin
  #1;
  mac_rx_mii = mii;
  mac_rx_frm = frm;
  mac_rx_pre = pre;
  mac_rx_sfd = sfd;
  mac_rx_dat = dat;
  mac_rx_fls = fls;
end
endtask


// ------------------------------------------------------------------------
// task: mac_rx_chk
//
// check rx event
task mac_rx_chk;
integer i;
reg m_end_seen;
begin
  m_end_seen = 1'b0;
  if ( mac_rx_frm )
  begin
    if(~mac_rx_mii)
    begin
      for ( mac_chk_ptr = 0; mac_chk_ptr < mac_rx_pre; mac_chk_ptr = mac_chk_ptr + 1 )
      begin
        mac_rx_data = mac_rx_mem[mac_chk_ptr];
        if ( mac_rx_data[7:0] !== 8'h55 )
        begin
          mac_rx_error_det = 1'b1;
          $display("%0d: ERROR: missing preamble byte %0d/%0d",$time, mac_chk_ptr,mac_rx_pre);
        end  
        else
          $display("%0d: preamble byte receieved %0d/%0d",$time, mac_chk_ptr,mac_rx_pre);
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
      end
      
      mac_rx_data = mac_rx_mem[mac_chk_ptr];
      if ( mac_rx_sfd ) 
        if ( mac_rx_data[7:0] !== 8'hD5 ) begin
          $display("%0d: ERROR: missing SFD",$time);
          mac_rx_error_det = 1'b1;
        end
        else
          $display("%0d:SFD receieved",$time);
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
      mac_chk_ptr = mac_chk_ptr + 1;

      for ( i = 0; i < mac_rx_dat; mac_chk_ptr = mac_chk_ptr + 1 )
      begin
        mac_rx_data = mac_rx_mem[mac_chk_ptr];
        if ( mac_rx_data[7:0] !== i[7:0] )
        begin
          $display("%0d: ERROR: wrong data byte %0d/%0d",$time,mac_chk_ptr,i);
          mac_rx_error_det = 1'b1;
        end  
        else
          $display("%0d:Data byte %0d/%0d",$time,mac_chk_ptr,i);
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
        i = i + 1;
      end
      
      if ( ~m_end_seen )
        for ( m_end_seen = 1'b0; mac_chk_ptr < 2048 && ~m_end_seen; mac_chk_ptr = mac_chk_ptr + 1 )
        begin
          mac_rx_data = mac_rx_mem[mac_chk_ptr];
          if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
        end  
      $display("%0d: MAC Rx %0dB frame (%0s)",$time, i, ( mac_rx_error_det ) ? "bad":"good");
      if(m_end_seen == 1'b1) rx_pkt_cnt= rx_pkt_cnt+1;
    end
    else
    begin
      for ( mac_chk_ptr = 0; mac_chk_ptr < mac_rx_pre; mac_chk_ptr = mac_chk_ptr + 1 )
      begin
        mac_rx_data = mac_rx_mem[mac_chk_ptr];
        if ( mac_rx_data[3:0] !== 4'h5 )
        begin
          mac_rx_error_det = 1'b1;
          $display("%0d: ERROR: missing preamble byte %0d/%0d rxed %d",$time, mac_chk_ptr,mac_rx_pre,mac_rx_data[3:0] );
          $display("debug=%0h",mac_rx_data[7:0]);
        end  
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
      end
      
      mac_rx_data = mac_rx_mem[mac_chk_ptr];
      if ( mac_rx_sfd ) 
        if ( mac_rx_data[3:0] !== 4'hD ) begin
          $display("%0d: ERROR: missing SFD byte %0d/%0d rxed %d ",$time,mac_chk_ptr,mac_rx_sfd, mac_rx_data[3:0]);
          mac_rx_error_det = 1'b1;
        end
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
      mac_chk_ptr = mac_chk_ptr + 1;

      for ( i = 0; i < mac_rx_dat; mac_chk_ptr = mac_chk_ptr + 1 )
      begin
        mac_rx_data = mac_rx_mem[mac_chk_ptr];
        if ( mac_rx_data[3:0] !== i[3:0] )
        begin
          $display("%0d: ERROR: wrong data byte %0d/%0d rxed %d ",$time,mac_chk_ptr,i,mac_rx_data[3:0]);
          mac_rx_error_det = 1'b1;
        end  
        if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
        i = i + 1;
      end
      
      if ( ~m_end_seen )
        for ( m_end_seen = 1'b0; mac_chk_ptr < 2048 && ~m_end_seen; mac_chk_ptr = mac_chk_ptr + 1 )
        begin
          mac_rx_data = mac_rx_mem[mac_chk_ptr];
          if ( ~mac_rx_data[10] ) m_end_seen = 1'b1;
        end  
      $display("%0d: MAC Rx %0dB frame (%0s)",$time, i, ( mac_rx_error_det ) ? "bad":"good");
      if(m_end_seen == 1'b1) rx_pkt_cnt= rx_pkt_cnt+1;
    end
  end
  else
  begin
    mac_rx_data = mac_rx_mem[0];
    if ( mac_rx_data[7:0] !== mac_rx_fls | ~mac_rx_data[9] ) begin
       mac_rx_error_det = 1'b1;
      $display("%0d: Warning: false carrier signalling %0h/%0h",$time, mac_rx_data[7:0],mac_rx_fls );
    end
    $display("%0d: MAC Detected rx_dv=%0h, rx_er=%0h, rxd=%0h",$time, mac_rx_data[8], mac_rx_data[9], mac_rx_data[7:0]);
  end
end
endtask


// ------------------------------------------------------------------------
// task: mac_pkt_wr
//
// writes a location of the packet memory
task mac_pkt_wr;
input  [15:0] addr;
input  [10:0] value;
begin
  mac_tx_mem[addr] = value;
end
endtask


// ------------------------------------------------------------------------
// task: mac_pkt_send
//
// sends packet contained in mac_tx_mem array
task mac_pkt_send;
begin
  @( gtx_clk ) #1;
  if ( gmii_en )
  begin
    $display( "%m ERROR: Tx packet already in progress." );
    $stop;
  end
  gmii_en = 1'b1;
  @( gtx_clk );
  @( negedge gmii_en ); // don't return from task until packet complete
end
endtask

reg [15:0] an_cfg_reg_val;
always @(posedge an_cfg_data_vld)
begin
    an_cfg_reg_val[15:0] = an_cfg_data[15:0];
//    $display("%t Config reg receieved and 16 bit value is 0x%x",$time,an_cfg_data);
end

initial
begin
  #100;
// ---------------------------------------------------------------------------
// Test script task calls
    reset;
    an_cfg_reg_val = 16'h0;
    test_anen_en_stat = 1'b0; 
    test_rx_slip_stat = 1'b0; 
    test_1g_loopback_stat = 1'b0;
    #5000;

    if(SLIP_ENABLE == 1) // PolarFire
      test_rx_slip();
    test_anen_en(1'b0);
    test_1g_loopback();
 
$display(" ---------------------------------------------------------");
$display(" TEST CASE STATUS:    ");
    if(SLIP_ENABLE == 1) // PolarFire
       $display (" RX_SLIP TEST : %s",(test_rx_slip_stat)?"PASS":"FAIL");
    $display (" AN TEST : %s",(test_anen_en_stat)?"PASS":"FAIL");
    $display (" 1000 MBPS LOOP BACK TEST : %s",(test_1g_loopback_stat)?"PASS":"FAIL");
$display(" ---------------------------------------------------------");
     
#5000;
$finish;
end

task test_rx_slip;
begin
// ------------------------------------------------------------------------
$display(" ---------------------------------------------------------");
$display(" - - - - -TEST CASE - RX_SLIP CHECK------ -  - - - - - - ");
$display(" ---------------------------------------------------------");
// ------------------------------------------------------------------------
  test_rx_slip_stat = 1'b0;
  
  // Shift the incoming data by 5 times before passing on to RCG and
  //  check decrease the shift in rcg by after receieveing a rx_slip
  num_of_slips = 5;
  rcg_shift = num_of_slips;

  // Apply phy reset for RX_SLIP SM
  mgmt_wr( phyid[4:0], 5'h00, 16'h8140 ); // rst,fdx,1000

  // Wait for few clock for phyreset and reset removal
  //   + SLIPs to be applied from DUT
  repeat(1500) @(posedge pma_rx_clk);

  if(rx_slip_cnt != num_of_slips) begin
       $display("ERROR: RX_SLIP :: num of slips rqrd %d & applied %d",num_of_slips,rx_slip_cnt);
       rx_slip_err = 1;
       end
  else begin
      $display("RX_SLIP :: num of slips rqrd %d & applied %d",num_of_slips,rx_slip_cnt);
      rx_slip_err = 0;
  end
  test_rx_slip_stat = (rx_slip_err == 1'b0);
end
endtask

task test_1g_loopback;
begin
// ------------------------------------------------------------------------
$display(" ---------------------------------------------------------");
$display(" - - - - -TEST CASE - 1000MBPS LOOP BACK -  - - - - - - ");
$display(" ---------------------------------------------------------");
// ------------------------------------------------------------------------
  test_1g_loopback_stat = 1'b0;
  mac_rx_error_det = 1'b0;
  mgmt_wr( phyid[4:0], 5'h00, 16'h8140 ); // rst,fdx,1000
  mgmt_wr( phyid[4:0], 5'h11, 16'h4008 ); // sltmr
  repeat(12) @ ( posedge mdc );
  mac_pkt_wr( 16'h0000, 11'h155 ); // /P
  mac_pkt_wr( 16'h0001, 11'h155 ); //
  mac_pkt_wr( 16'h0002, 11'h155 ); //
  mac_pkt_wr( 16'h0003, 11'h155 ); //
  mac_pkt_wr( 16'h0004, 11'h155 ); //
  mac_pkt_wr( 16'h0005, 11'h155 ); //
  mac_pkt_wr( 16'h0006, 11'h155 ); //
  mac_pkt_wr( 16'h0007, 11'h155 ); //
  mac_pkt_wr( 16'h0008, 11'h1D5 ); // /SFD
  mac_pkt_wr( 16'h0009, { 3'h1, 8'h00 }); // 
  mac_pkt_wr( 16'h000a, { 3'h1, 8'h01 });
  mac_pkt_wr( 16'h000b, { 3'h1, 8'h02 });
  mac_pkt_wr( 16'h000c, { 3'h1, 8'h03 });
  mac_pkt_wr( 16'h000d, { 3'h1, 8'h04 });
  mac_pkt_wr( 16'h000e, { 3'h1, 8'h05 });
  mac_pkt_wr( 16'h000f, { 3'h1, 8'h06 });
  mac_pkt_wr( 16'h0010, { 3'h1, 8'h07 });
  mac_pkt_wr( 16'h0011, { 3'h5, 8'h08 });
  mac_pkt_send;

  mac_rx_exp(1'b0, 1'b1, 4'h8, 1'b1, 16'h9, 8'h0 );

  repeat(50)  @ ( posedge pma_rx_clk );
  test_1g_loopback_stat = ((mac_rx_error_det == 1'b0) && (rx_pkt_cnt > 'h0));
  if ( test_1g_loopback_stat == 1'b0 )
  begin
    if(rx_pkt_cnt == 0)
      $display("\n%0d: ERROR: No packet Receieved .",$time);
    else
      $display("\n%0d: ERROR: Receieved packet is wrong.",$time);
  end
  else 
    $display("\n%0d: PASS: Receieved packet is as expected.",$time);
end
endtask


// ------------------------------------------------------------------------
// Auto-Negotiation Enabled
// TEST SEQUENCE
//1.Initialize by restarting auto-negotiation by writing to register. 
//2. Verify that Autoneogation was successful. 

task test_anen_en(input reg rst_an_only);
reg    [15:0] ii, mgmt_read;
begin
// ------------------------------------------------------------------------
$display(" ---------------------------------------------------------");
$display(" - - - - -TEST CASE AUTO NEGOTIATION ENABLE  - - - - - - ");
$display(" ---------------------------------------------------------");
// ------------------------------------------------------------------------
  test_anen_en_stat = 1'b0;
  anx_state_rst = 1;
  repeat(50)  @ ( posedge pma_rx_clk );
  anx_state_rst = 0;
  $display("\n Auto-Negotiation intiated by CoreSGMII Module");
  $display("\n%0d: Step 1: Restart Auto Negotiation",$time);
  if(rst_an_only == 1'b0)
  mgmt_wr( phyid[4:0], 5'h11, 16'h4008 ); // sltmr
  mgmt_wr( phyid[4:0], 5'h04, 16'h00A0 ); // anar (NP=0 in base page)
  mgmt_wr( phyid[4:0], 5'h00, 16'h9140 ); // rstphy,anen,fdx,1000
  mgmt_wr( phyid[4:0], 5'h00, 16'h1340 ); // anen, rstan,fdx,1000
 repeat(10000)  @ ( posedge pma_rx_clk );

  $display("\n%0d: Step 4: Verifying Auto-negotiation Completed", $time);
  mgmt_rd( phyid[4:0], 5'h01, mgmt_read );
  test_anen_en_stat = 1'b1;
  if ( an_cfg_reg_val[15:0] == 'h0 )
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation config reg rxed is zero",$time);
  end
  if ( mgmt_read[5] !== 1'b1 )
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation did not Complete",$time);
  end

  if(anx_cfg_reg_cnt == 'h0)
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation config reg receieved count is zero",$time);
  end
       
  //ANX STATE MACHINE STATUS CHECK
  if(anx_state[8] === 'b0)
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation state machine status port ANX_STATE[8] (LINK OK STATE) is LOW",$time);
  end

  if(anx_state[0] === 'b1)
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation state machine status port ANX_STATE[0] (DIS OK STATE) is HIGH",$time);
  end

  if(anx_latched_stat[7:1] === 6'b0)
  begin
    test_anen_en_stat = 1'b0;
    $display("\n%0d: ERROR: Auto-negotiation state machine did not entered clause 37 define AN states",$time);
  end
  if ( mgmt_read[5] == 1'b1 )
  begin
    if(test_anen_en_stat == 1'b1) 
      $display("\n%0d: PASS: Auto-negotiation Completed",$time);
  end
end
endtask
endmodule
